Module: OpenC3::Extract

Included in:
Api, ApiShared, Script
Defined in:
lib/openc3/script/extract.rb

Constant Summary collapse

SCANNING_REGULAR_EXPRESSION =

Tokenizer for command parameters: matches double quoted strings, single quoted strings, bracket delimited arrays (one level of nesting), a bare comma delimiter, or bare words (runs of non-whitespace, non-comma characters). Commas are tokenized separately so whitespace around them is optional.

%r{ (?:"(?:[^\\"]|\\.)*") | (?:'(?:[^\\']|\\.)*') | (?:\[(?:[^\\\[\]]|\\.|\[(?:[^\\\[\]]|\\.)*\])*\]) | , | [^\s,]+ }x