Module: OpenC3::HistoryCountIdentifier

Overview

Identifier for channels whose only parameter is the event history count. Including classes supply the channel name as a CHANNEL constant.

Instance Method Summary collapse

Instance Method Details

#initialize(history_count: 0, **options) ⇒ Object



264
265
266
267
268
269
270
# File 'lib/openc3/script/web_socket_api.rb', line 264

def initialize(history_count: 0, **options)
  @identifier = {
    channel: self.class::CHANNEL,
    history_count: history_count
  }
  super(**options)
end