Method: OpenC3::HttpClientInterface#read_interface

Defined in:
lib/openc3/interfaces/http_client_interface.rb

#read_interfaceObject

Returns the response data and extra from the interface which was queued up by the write_interface method. Read protocols can then potentially modify the data in their read_data methods. Then convert_data_to_packet is called to convert the data into a Packet object. Finally the read protocols read_packet methods are called.



174
175
176
177
178
179
180
181
# File 'lib/openc3/interfaces/http_client_interface.rb', line 174

def read_interface
  # This blocks until a response is available
  data, extra = @response_queue.pop
  return nil if data.nil?

  read_interface_base(data, extra)
  return data, extra
end