Class: OpenC3::MigrationModel
- Defined in:
- lib/openc3/models/migration_model.rb
Constant Summary collapse
- PRIMARY_KEY =
'openc3__migrations'
Instance Attribute Summary
Attributes inherited from Model
#name, #plugin, #scope, #updated_at
Class Method Summary collapse
- .all(scope: nil) ⇒ Object
-
.get(name:, scope: nil) ⇒ Object
NOTE: The following three class methods are used by the ModelController and are reimplemented to enable various Model class methods to work.
- .names(scope: nil) ⇒ Object
Instance Method Summary collapse
-
#as_json(*a) ⇒ Hash
JSON encoding of this model.
-
#initialize(name:) ⇒ MigrationModel
constructor
END NOTE.
Methods inherited from Model
#check_disable_erb, #create, #deploy, #destroy, #destroyed?, #diff, filter, find_all_by_plugin, from_json, get_all_models, get_model, handle_config, set, store, store_queued, #undeploy, #update
Constructor Details
#initialize(name:) ⇒ MigrationModel
END NOTE
35 36 37 |
# File 'lib/openc3/models/migration_model.rb', line 35 def initialize(name:) super(PRIMARY_KEY, name: name) end |
Class Method Details
.all(scope: nil) ⇒ Object
30 31 32 |
# File 'lib/openc3/models/migration_model.rb', line 30 def self.all(scope: nil) super(PRIMARY_KEY) end |
.get(name:, scope: nil) ⇒ Object
NOTE: The following three class methods are used by the ModelController and are reimplemented to enable various Model class methods to work
22 23 24 |
# File 'lib/openc3/models/migration_model.rb', line 22 def self.get(name:, scope: nil) super(PRIMARY_KEY, name: name) end |
.names(scope: nil) ⇒ Object
26 27 28 |
# File 'lib/openc3/models/migration_model.rb', line 26 def self.names(scope: nil) super(PRIMARY_KEY) end |
Instance Method Details
#as_json(*a) ⇒ Hash
Returns JSON encoding of this model.
40 41 42 43 44 45 |
# File 'lib/openc3/models/migration_model.rb', line 40 def as_json(*a) { 'name' => @name, 'updated_at' => @updated_at } end |