Method: OpenC3::SerialStream#read
- Defined in:
- lib/openc3/streams/serial_stream.rb
#read ⇒ String
Returns a binary string of data from the serial port
143 144 145 146 147 148 |
# File 'lib/openc3/streams/serial_stream.rb', line 143 def read raise "Attempt to read from write only stream" unless @read_serial_port # No read mutex is needed because reads happen serially @read_serial_port.read end |