Module: OpenC3::Extract
Constant Summary collapse
- SCANNING_REGULAR_EXPRESSION =
Tokenizer for command parameters: matches double quoted strings, single quoted strings, bracket delimited arrays (one level of nesting), a bare comma delimiter, or bare words (runs of non-whitespace, non-comma characters). Commas are tokenized separately so whitespace around them is optional.
%r{ (?:"(?:[^\\"]|\\.)*") | (?:'(?:[^\\']|\\.)*') | (?:\[(?:[^\\\[\]]|\\.|\[(?:[^\\\[\]]|\\.)*\])*\]) | , | [^\s,]+ }x