Method: OpenC3::PacketConfig#limits_groups
- Defined in:
- lib/openc3/packets/packet_config.rb
#limits_groups ⇒ Hash<String=>Array(String, String, String)> (readonly)
Returns Hash of all the limits groups keyed by the group name. The value is a three element array consisting of the target_name, packet_name, and item_name.
57 58 59 |
# File 'lib/openc3/packets/packet_config.rb', line 57 def limits_groups @limits_groups end |