Class: OpenC3::ScopeModel
- Inherits:
-
Model
show all
- Defined in:
- lib/openc3/models/scope_model.rb
Constant Summary
collapse
- PRIMARY_KEY =
"openc3_scopes"
- SYSTEM_HEALTH_DEFAULTS =
Default thresholds used by the System Health tool and the metrics
microservices. Stored in the 'system_health' setting as a JSON string.
{
"cpu" => {
"redThreshold" => 90.0,
"yellowThreshold" => 80.0,
"snoozeMinutes" => 15,
"lastTriggerTimeRed" => nil, "lastTriggerTimeYellow" => nil, "sustainedSeconds" => 15
},
"memory" => {
"redThreshold" => 90.0,
"yellowThreshold" => 80.0,
"snoozeMinutes" => 15,
"lastTriggerTimeRed" => nil, "lastTriggerTimeYellow" => nil, "sustainedSeconds" => 15
},
"disk" => {
"redThreshold" => 90.0,
"yellowThreshold" => 80.0,
"snoozeMinutes" => 720, "lastTriggerTimeRed" => nil, "lastTriggerTimeYellow" => nil, "sustainedSeconds" => 60
},
"global" => {
"enableAlerts" => true
}
}
Instance Attribute Summary collapse
Attributes inherited from Model
#name, #plugin, #scope, #updated_at
Class Method Summary
collapse
Instance Method Summary
collapse
-
#as_json(*_a) ⇒ Object
-
#create(update: false, force: false, queued: false) ⇒ Object
-
#deploy(gem_path, variables) ⇒ Object
-
#deploy_critical_cmd_microservice(gem_path, variables, parent) ⇒ Object
-
#deploy_openc3_log_messages_microservice(gem_path, variables, parent) ⇒ Object
-
#deploy_periodic_microservice(gem_path, variables, parent) ⇒ Object
-
#deploy_scopecleanup_microservice(gem_path, variables, parent) ⇒ Object
-
#deploy_scopemulti_microservice(gem_path, variables) ⇒ Object
-
#deploy_unknown_commandlog_microservice(gem_path, variables, parent) ⇒ Object
-
#deploy_unknown_packetlog_microservice(gem_path, variables, parent) ⇒ Object
-
#destroy ⇒ Object
-
#initialize(name:, text_log_cycle_time: 600, text_log_cycle_size: 50_000_000, text_log_retain_time: nil, tool_log_retain_time: nil, cleanup_poll_time: 3600, command_authority: false, critical_commanding: "OFF", shard: 0, updated_at: nil) ⇒ ScopeModel
constructor
A new instance of ScopeModel.
-
#seed_database ⇒ Object
-
#undeploy ⇒ Object
Methods inherited from Model
#check_disable_erb, #destroyed?, #diff, filter, find_all_by_plugin, get_all_models, handle_config, set, store, store_queued, #update
Constructor Details
#initialize(name:, text_log_cycle_time: 600, text_log_cycle_size: 50_000_000, text_log_retain_time: nil, tool_log_retain_time: nil, cleanup_poll_time: 3600, command_authority: false, critical_commanding: "OFF", shard: 0, updated_at: nil) ⇒ ScopeModel
Returns a new instance of ScopeModel.
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
|
# File 'lib/openc3/models/scope_model.rb', line 120
def initialize(name:,
text_log_cycle_time: 600,
text_log_cycle_size: 50_000_000,
text_log_retain_time: nil,
tool_log_retain_time: nil,
cleanup_poll_time: 3600,
command_authority: false,
critical_commanding: "OFF",
shard: 0,
updated_at: nil)
super(
PRIMARY_KEY,
name: name,
updated_at: updated_at,
scope: name
)
@text_log_cycle_time = text_log_cycle_time
@text_log_cycle_size = text_log_cycle_size
@text_log_retain_time = text_log_retain_time
@tool_log_retain_time = tool_log_retain_time
@cleanup_poll_time = cleanup_poll_time
@command_authority = command_authority
@critical_commanding = critical_commanding.to_s.upcase
@critical_commanding = "OFF" if @critical_commanding.length == 0
if !["OFF", "NORMAL", "ALL"].include?(@critical_commanding)
raise "Invalid value for critical_commanding: #{@critical_commanding}"
end
@shard = shard.to_i @children = []
end
|
Instance Attribute Details
Returns the value of attribute children.
78
79
80
|
# File 'lib/openc3/models/scope_model.rb', line 78
def children
@children
end
|
#cleanup_poll_time ⇒ Object
Returns the value of attribute cleanup_poll_time.
83
84
85
|
# File 'lib/openc3/models/scope_model.rb', line 83
def cleanup_poll_time
@cleanup_poll_time
end
|
#command_authority ⇒ Object
Returns the value of attribute command_authority.
84
85
86
|
# File 'lib/openc3/models/scope_model.rb', line 84
def command_authority
@command_authority
end
|
#critical_commanding ⇒ Object
Returns the value of attribute critical_commanding.
85
86
87
|
# File 'lib/openc3/models/scope_model.rb', line 85
def critical_commanding
@critical_commanding
end
|
Returns the value of attribute shard.
86
87
88
|
# File 'lib/openc3/models/scope_model.rb', line 86
def shard
@shard
end
|
#text_log_cycle_size ⇒ Object
Returns the value of attribute text_log_cycle_size.
80
81
82
|
# File 'lib/openc3/models/scope_model.rb', line 80
def text_log_cycle_size
@text_log_cycle_size
end
|
#text_log_cycle_time ⇒ Object
Returns the value of attribute text_log_cycle_time.
79
80
81
|
# File 'lib/openc3/models/scope_model.rb', line 79
def text_log_cycle_time
@text_log_cycle_time
end
|
#text_log_retain_time ⇒ Object
Returns the value of attribute text_log_retain_time.
81
82
83
|
# File 'lib/openc3/models/scope_model.rb', line 81
def text_log_retain_time
@text_log_retain_time
end
|
Returns the value of attribute tool_log_retain_time.
82
83
84
|
# File 'lib/openc3/models/scope_model.rb', line 82
def tool_log_retain_time
@tool_log_retain_time
end
|
Class Method Details
.all(scope: nil) ⇒ Object
101
102
103
|
# File 'lib/openc3/models/scope_model.rb', line 101
def self.all(scope: nil)
super(PRIMARY_KEY)
end
|
.from_json(json, scope: nil) ⇒ Object
105
106
107
108
109
|
# File 'lib/openc3/models/scope_model.rb', line 105
def self.from_json(json, scope: nil)
json = JSON.parse(json, allow_nan: true, create_additions: true) if String === json
raise "json data is nil" if json.nil?
new(**json.transform_keys(&:to_sym))
end
|
.get(name:, scope: nil) ⇒ Object
NOTE: The following three class methods are used by the ModelController
and are reimplemented to enable various Model class methods to work
The scope keyword is given to support the ModelController method signature
even though it is not used
93
94
95
|
# File 'lib/openc3/models/scope_model.rb', line 93
def self.get(name:, scope: nil)
super(PRIMARY_KEY, name: name)
end
|
.get_model(name:, scope: nil) ⇒ Object
111
112
113
114
115
116
117
118
|
# File 'lib/openc3/models/scope_model.rb', line 111
def self.get_model(name:, scope: nil)
json = get(name: name)
if json
from_json(json)
else
nil
end
end
|
.names(scope: nil) ⇒ Object
97
98
99
|
# File 'lib/openc3/models/scope_model.rb', line 97
def self.names(scope: nil)
super(PRIMARY_KEY)
end
|
Instance Method Details
#as_json(*_a) ⇒ Object
202
203
204
205
206
207
208
209
210
211
212
213
|
# File 'lib/openc3/models/scope_model.rb', line 202
def as_json(*_a)
{"name" => @name,
"updated_at" => @updated_at,
"text_log_cycle_time" => @text_log_cycle_time,
"text_log_cycle_size" => @text_log_cycle_size,
"text_log_retain_time" => @text_log_retain_time,
"tool_log_retain_time" => @tool_log_retain_time,
"cleanup_poll_time" => @cleanup_poll_time,
"command_authority" => @command_authority,
"critical_commanding" => @critical_commanding,
"shard" => @shard}
end
|
#create(update: false, force: false, queued: false) ⇒ Object
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
|
# File 'lib/openc3/models/scope_model.rb', line 153
def create(update: false, force: false, queued: false)
raise "Invalid scope name: #{@name}" if !/^[a-zA-Z0-9_-]+$/.match?(@name)
@name = @name.upcase
@scope = @name @text_log_cycle_time = @text_log_cycle_time.to_i
@text_log_cycle_size = @text_log_cycle_size.to_i
@text_log_retain_time = @text_log_retain_time.to_i if @text_log_retain_time
@tool_log_retain_time = @tool_log_retain_time.to_i if @tool_log_retain_time
@cleanup_poll_time = @cleanup_poll_time.to_i
super
if ENTERPRISE
if update and @command_authority == false
CmdAuthorityModel.names(scope: @name).each do |auth_name|
model = CmdAuthorityModel.get_model(name: auth_name, scope: @name)
model.destroy if model
end
end
if update and @critical_commanding == "OFF"
CriticalCmdModel.names(scope: @name).each do |name|
model = CriticalCmdModel.get_model(name: name, scope: @name)
model.destroy if model
end
end
end
SystemEventsTopic.write(:scope, as_json)
end
|
#deploy(gem_path, variables) ⇒ Object
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
|
# File 'lib/openc3/models/scope_model.rb', line 350
def deploy(gem_path, variables)
seed_database
if ENTERPRISE
model = TriggerGroupModel.get(name: "DEFAULT", scope: @scope)
unless model
model = TriggerGroupModel.new(name: "DEFAULT", shard: @shard, scope: @scope)
model.create
model.deploy
end
end
model = TargetModel.new(name: "UNKNOWN", shard: @shard, scope: @scope)
model.create
@parent = "#{@scope}__SCOPEMULTI__#{@scope}"
deploy_openc3_log_messages_microservice(gem_path, variables, @parent)
deploy_unknown_commandlog_microservice(gem_path, variables, @parent)
deploy_unknown_packetlog_microservice(gem_path, variables, @parent)
if @scope == "DEFAULT"
deploy_periodic_microservice(gem_path, variables, @parent)
end
deploy_scopecleanup_microservice(gem_path, variables, @parent)
if ENTERPRISE
deploy_critical_cmd_microservice(gem_path, variables, @parent)
end
deploy_scopemulti_microservice(gem_path, variables)
end
|
#deploy_critical_cmd_microservice(gem_path, variables, parent) ⇒ Object
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
|
# File 'lib/openc3/models/scope_model.rb', line 319
def deploy_critical_cmd_microservice(gem_path, variables, parent)
microservice_name = "#{@scope}__CRITICALCMD__#{@scope}"
microservice = MicroserviceModel.new(
name: microservice_name,
cmd: ["ruby", "critical_cmd_microservice.rb", microservice_name],
work_dir: "/openc3-enterprise/lib/openc3-enterprise/microservices",
parent: parent,
shard: @shard,
scope: @scope
)
microservice.create
microservice.deploy(gem_path, variables)
@children << microservice_name if parent
Logger.info "Configured microservice #{microservice_name}"
end
|
#deploy_openc3_log_messages_microservice(gem_path, variables, parent) ⇒ Object
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
|
# File 'lib/openc3/models/scope_model.rb', line 215
def deploy_openc3_log_messages_microservice(gem_path, variables, parent)
microservice_name = "#{@scope}__OPENC3__LOG"
topics = ["#{@scope}__openc3_log_messages"]
if @scope == "DEFAULT"
topics << "NOSCOPE__openc3_log_messages"
end
microservice = MicroserviceModel.new(
name: microservice_name,
cmd: ["ruby", "text_log_microservice.rb", microservice_name],
work_dir: "/openc3/lib/openc3/microservices",
options: [
["CYCLE_TIME", @text_log_cycle_time],
["CYCLE_SIZE", @text_log_cycle_size]
],
topics: topics,
parent: parent,
shard: @shard,
scope: @scope
)
microservice.create
microservice.deploy(gem_path, variables)
@children << microservice_name if parent
Logger.info "Configured microservice #{microservice_name}"
end
|
#deploy_periodic_microservice(gem_path, variables, parent) ⇒ Object
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
|
# File 'lib/openc3/models/scope_model.rb', line 287
def deploy_periodic_microservice(gem_path, variables, parent)
microservice_name = "#{@scope}__PERIODIC__#{@scope}"
microservice = MicroserviceModel.new(
name: microservice_name,
cmd: ["ruby", "periodic_microservice.rb", microservice_name],
work_dir: "/openc3/lib/openc3/microservices",
parent: parent,
shard: @shard,
scope: @scope
)
microservice.create
microservice.deploy(gem_path, variables)
@children << microservice_name if parent
Logger.info "Configured microservice #{microservice_name}"
end
|
#deploy_scopecleanup_microservice(gem_path, variables, parent) ⇒ Object
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
|
# File 'lib/openc3/models/scope_model.rb', line 303
def deploy_scopecleanup_microservice(gem_path, variables, parent)
microservice_name = "#{@scope}__SCOPECLEANUP__#{@scope}"
microservice = MicroserviceModel.new(
name: microservice_name,
cmd: ["ruby", "scope_cleanup_microservice.rb", microservice_name],
work_dir: "/openc3/lib/openc3/microservices",
parent: parent,
shard: @shard,
scope: @scope
)
microservice.create
microservice.deploy(gem_path, variables)
@children << microservice_name if parent
Logger.info "Configured microservice #{microservice_name}"
end
|
#deploy_scopemulti_microservice(gem_path, variables) ⇒ Object
335
336
337
338
339
340
341
342
343
344
345
346
347
348
|
# File 'lib/openc3/models/scope_model.rb', line 335
def deploy_scopemulti_microservice(gem_path, variables)
microservice_name = "#{@scope}__SCOPEMULTI__#{@scope}"
microservice = MicroserviceModel.new(
name: microservice_name,
cmd: ["ruby", "multi_microservice.rb", *@children],
work_dir: "/openc3/lib/openc3/microservices",
target_names: [],
shard: @shard,
scope: @scope
)
microservice.create
microservice.deploy(gem_path, variables)
Logger.info "Configured microservice #{microservice_name}"
end
|
#deploy_unknown_commandlog_microservice(gem_path, variables, parent) ⇒ Object
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
|
# File 'lib/openc3/models/scope_model.rb', line 241
def deploy_unknown_commandlog_microservice(gem_path, variables, parent)
microservice_name = "#{@scope}__COMMANDLOG__UNKNOWN"
microservice = MicroserviceModel.new(
name: microservice_name,
cmd: ["ruby", "log_microservice.rb", microservice_name],
work_dir: "/openc3/lib/openc3/microservices",
options: [
["RAW_OR_DECOM", "RAW"],
["CMD_OR_TLM", "CMD"],
["CYCLE_TIME", "3600"] ],
topics: ["#{@scope}__COMMAND__{UNKNOWN}__UNKNOWN"],
target_names: [],
parent: parent,
shard: @shard,
scope: @scope
)
microservice.create
microservice.deploy(gem_path, variables)
@children << microservice_name if parent
Logger.info "Configured microservice #{microservice_name}"
end
|
#deploy_unknown_packetlog_microservice(gem_path, variables, parent) ⇒ Object
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
|
# File 'lib/openc3/models/scope_model.rb', line 264
def deploy_unknown_packetlog_microservice(gem_path, variables, parent)
microservice_name = "#{@scope}__PACKETLOG__UNKNOWN"
microservice = MicroserviceModel.new(
name: microservice_name,
cmd: ["ruby", "log_microservice.rb", microservice_name],
work_dir: "/openc3/lib/openc3/microservices",
options: [
["RAW_OR_DECOM", "RAW"],
["CMD_OR_TLM", "TLM"],
["CYCLE_TIME", "3600"] ],
topics: ["#{@scope}__TELEMETRY__{UNKNOWN}__UNKNOWN"],
target_names: [],
parent: parent,
shard: @shard,
scope: @scope
)
microservice.create
microservice.deploy(gem_path, variables)
@children << microservice_name if parent
Logger.info "Configured microservice #{microservice_name}"
end
|
189
190
191
192
193
194
195
196
197
198
199
200
|
# File 'lib/openc3/models/scope_model.rb', line 189
def destroy
if @name != "DEFAULT"
plugins = PluginModel.get_all_models(scope: @name)
plugins.each do |_plugin_name, plugin|
plugin.destroy
end
super
else
raise "DEFAULT scope cannot be destroyed"
end
end
|
#seed_database ⇒ Object
431
432
433
434
435
436
437
438
439
440
441
442
443
444
|
# File 'lib/openc3/models/scope_model.rb', line 431
def seed_database
setting = SettingModel.get(name: "source_url")
SettingModel.set({name: "source_url", data: "https://github.com/OpenC3/cosmos"}, scope: @scope) unless setting
setting = SettingModel.get(name: "rubygems_url")
SettingModel.set({name: "rubygems_url", data: ENV["RUBYGEMS_URL"] || "https://rubygems.org"}, scope: @scope) unless setting
setting = SettingModel.get(name: "pypi_url")
SettingModel.set({name: "pypi_url", data: ENV["PYPI_URL"] || "https://pypi.org"}, scope: @scope) unless setting
SettingModel.set({name: "news_feed", data: true}, scope: @scope)
setting = SettingModel.get(name: "system_health")
SettingModel.set({name: "system_health", data: JSON.generate(SYSTEM_HEALTH_DEFAULTS)}, scope: "DEFAULT") unless setting
end
|
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
|
# File 'lib/openc3/models/scope_model.rb', line 396
def undeploy
target = TargetModel.get_model(name: "UNKNOWN", scope: @scope)
target.destroy
model = MicroserviceModel.get_model(name: "#{@scope}__SCOPEMULTI__#{@scope}", scope: @scope)
model.destroy if model
model = MicroserviceModel.get_model(name: "#{@scope}__SCOPECLEANUP__#{@scope}", scope: @scope)
model.destroy if model
model = MicroserviceModel.get_model(name: "#{@scope}__OPENC3__LOG", scope: @scope)
model.destroy if model
model = MicroserviceModel.get_model(name: "#{@scope}__COMMANDLOG__UNKNOWN", scope: @scope)
model.destroy if model
model = MicroserviceModel.get_model(name: "#{@scope}__PACKETLOG__UNKNOWN", scope: @scope)
model.destroy if model
model = MicroserviceModel.get_model(name: "#{@scope}__PERIODIC__#{@scope}", scope: @scope)
model.destroy if model
if ENTERPRISE
model = MicroserviceModel.get_model(name: "#{@scope}__TRIGGER_GROUP__DEFAULT", scope: @scope)
model.destroy if model
model = MicroserviceModel.get_model(name: "#{@scope}__CRITICALCMD__#{@scope}", scope: @scope)
model.destroy if model
Topic.del("#{@scope}__openc3_autonomic")
Topic.del("#{@scope}__TRIGGER__GROUP")
end
db_shard = Store.db_shard_for_target('UNKNOWN', scope: @scope)
Topic.del("#{@scope}__COMMAND__{UNKNOWN}__UNKNOWN", db_shard: db_shard)
Topic.del("#{@scope}__TELEMETRY__{UNKNOWN}__UNKNOWN", db_shard: db_shard)
Topic.del("#{@scope}__openc3_targets")
Topic.del("#{@scope}__CONFIG")
end
|